@@ -1,9 +1,10 @@
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings, BangPatterns #-}
 
 module Network.Wai.Application.Classic.Path (
-    Path(..)
-  , fromString, fromByteString
-  , (+++), (</>), (<\>), (<.>)
+    Path
+  , pathString
+  , fromString
+  , (</>), (<\>), (<.>)
   , breakAtSeparator, hasLeadingPathSeparator, hasTrailingPathSeparator
   , isSuffixOf
   ) where
@@ -11,38 +12,20 @@ module Network.Wai.Application.Classic.Path (
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as B8
-import Data.Function
 import Data.String
 import Data.Word
 
 ----------------------------------------------------------------
 
--- | Smart file path.
-data Path = Path {
-    pathString :: FilePath
-  , pathByteString :: ByteString
-  }
+-- | File path.
+type Path = ByteString
 
-instance IsString Path where
-    fromString path = Path {
-        pathString = path
-      , pathByteString = B8.pack path
-      }
-
-instance Show Path where
-    show = show . pathByteString
-
-instance Eq Path where
-    (==) = (==) `on` pathByteString
+pathString :: Path -> String
+pathString = B8.unpack
+{-# INLINE pathString #-}
 
 ----------------------------------------------------------------
 
-fromByteString :: ByteString -> Path
-fromByteString path = Path {
-    pathString = B8.unpack path
-  , pathByteString = path
-  }
-
 -- pathDot :: Word8
 -- pathDot = 46
 
@@ -64,12 +47,11 @@ True
 False
 -}
 hasLeadingPathSeparator :: Path -> Bool
-hasLeadingPathSeparator path
+hasLeadingPathSeparator bs
   | BS.null bs            = False
   | BS.head bs == pathSep = True
   | otherwise             = False
-  where
-    bs = pathByteString path
+{-# INLINE hasLeadingPathSeparator #-}
 
 {-|
   Checking if the path ends with the path separator.
@@ -80,23 +62,11 @@ True
 False
 -}
 hasTrailingPathSeparator :: Path -> Bool
-hasTrailingPathSeparator path
+hasTrailingPathSeparator bs
   | BS.null bs            = False
   | BS.last bs == pathSep = True
   | otherwise             = False
-  where
-    bs = pathByteString path
-
-infixr +++
-
-{-|
-  Appending.
--}
-
-(+++) :: Path -> Path -> Path
-p1 +++ p2 = fromByteString p
-  where
-    p = pathByteString p1 `BS.append` pathByteString p2
+{-# INLINE hasTrailingPathSeparator #-}
 
 {-|
   Appending with the file separator.
@@ -112,17 +82,15 @@ p1 +++ p2 = fromByteString p
 -}
 
 (</>) :: Path -> Path -> Path
-p1 </> p2
-  | has1 && not has2 || not has1 && has2 = p1 +++ p2
-  | has1      = fromByteString pp1
-  | otherwise = fromByteString pp2
+p1 </> p2 = p
   where
-    has1 = hasTrailingPathSeparator p1
-    has2 = hasLeadingPathSeparator p2
-    p1' = pathByteString p1
-    p2' = pathByteString p2
-    pp1 = p1' `BS.append` BS.tail p2'
-    pp2 = BS.concat [p1',pathSepBS,p2']
+    !has1 = hasTrailingPathSeparator p1
+    !has2 = hasLeadingPathSeparator p2
+    !p | has1 && not has2 = p1 `BS.append` p2
+       | not has1 && has2 = p1 `BS.append` p2
+       | has1             = p1 `BS.append` BS.tail p2
+       | otherwise        = BS.concat [p1,pathSepBS,p2]
+{-# INLINE (</>) #-}
 
 {-|
   Removing prefix. The prefix of the second argument is removed
@@ -136,21 +104,19 @@ p1 </> p2
 "bar"
 -}
 (<\>) :: Path -> Path -> Path
-p1 <\> p2 = fromByteString p
+p1 <\> p2 = p
   where
-    p1' = pathByteString p1
-    p2' = pathByteString p2
-    p = BS.drop (BS.length p2') p1'
+    !p = BS.drop (BS.length p2) p1
+{-# INLINE (<\>) #-}
 
 {-|
   Adding suffix.
 -}
 (<.>) :: Path -> Path -> Path
-p1 <.> p2 = fromByteString p
+p1 <.> p2 = p
   where
-    p1' = pathByteString p1
-    p2' = pathByteString p2
-    p = BS.concat [p1',pathDotBS,p2']
+    !p = BS.concat [p1,pathDotBS,p2]
+{-# INLINE (<.>) #-}
 
 {-|
   Breaking at the first path separator.
@@ -163,13 +129,9 @@ p1 <.> p2 = fromByteString p
 ("foo","")
 -}
 breakAtSeparator :: Path -> (Path,Path)
-breakAtSeparator p = (fromByteString r1, fromByteString r2)
-  where
-    p' = pathByteString p
-    (r1,r2) = BS.break (== pathSep) p'
+breakAtSeparator p = BS.break (== pathSep) p
+{-# INLINE breakAtSeparator #-}
 
 isSuffixOf :: Path -> Path -> Bool
-isSuffixOf p1 p2 = p1' `BS.isSuffixOf` p2'
-  where
-    p1' = pathByteString p1
-    p2' = pathByteString p2
+isSuffixOf = BS.isSuffixOf
+{-# INLINE isSuffixOf #-}
