@@ -15,11 +15,11 @@ redirectApp :: ClassicAppSpec -> RedirectRoute -> Application
 redirectApp _ route req respond =
     respond $ responseLBS status hdr ""
   where
-    path = fromByteString $ rawPathInfo req
+    path = rawPathInfo req
     src = redirectSrc route
     dst = redirectDst route
     -- Scheme must not be included because of no way to tell
     -- http or https.
-    rurl = "//" `append` pathByteString (dst </> (path <\> src))
+    rurl = "//" `append` (dst </> (path <\> src))
     hdr = locationHeader rurl
     status = movedPermanently301
